(******************************************************************
  dx - Convert DOS / UNIX textfiles, GPL by Arne Hanssen
 -----------------------------------------------------------------
  11.06.99 Created: Arne Hanssen (my first real Linux/FPC program)
  21.09.99 Unix-to-DOS conv. implemented, AHH
  25.09.99 Added option to force resulttupe, cleaned up code, AHH
           Outfile now optional, converting sourcefile directly
  26.09.99 Option to query source file format, more code cleanup
           Read source into memory, avoid tempfile if no target
  27.09.99 Bugfix: InFName could be interpreted as option
                   Forgot to release buffer memory
 -----------------------------------------------------------------
  TODO:    Linux version might use LINUX unit???
  BUGS:    Not that I know ;-)
******************************************************************)

Program DosUnixText;


type
  TxtFType = ( ftUnknown, ftQueried, ftDosText, ftUnixText );


const
  LF = #10;
  CR = #13;


Procedure ShowUsage;
begin
  writeln;
  writeln('Syntax: ', ParamStr( 0 ), ' [-opt] <infilename> [<outfilename>]');
  writeln;
  writeln(#9'option:');
  writeln(#9#9'-q  query source file type (no action performed)');
  writeln(#9#9'-d  make a DOS-type file');
  writeln(#9#9'-u  make a UNIX-type file');
  writeln(#9#9'    if no option, file is converted to "the other type"');
  writeln;
end;


Procedure ProsessParams( var OutType: TxtFType; var InFName, OutFName: string );
var
  i: integer;
  Param: string;
begin
  InFName  := '';
  OutFName := '';
  OutType := ftUnknown;
  for i := 1 to ParamCount do begin
    Param := ParamStr( i );
    if ( Param[1] = '-' ) and ( i = 1 ) and ( Length( Param ) = 2 )
     then case UpCase( Param[ 2 ] ) of
      'D': OutType := ftDosText;
      'U': OutType := ftUnixText;
      'Q': OutType := ftQueried;
    end
    else begin
      if InFName = '' then InFName := Param
      else if OutFName = '' then OutFName := Param;
    end;
  end;
end;


Procedure DetectType( var InType: TxtFType; const FBuff: PChar; const FSize: LongInt );
var
  chIx: LongInt;
begin
  InType := ftUnknown;
  chIx := 0;
  while ( chIx < FSize ) and ( InType = ftUnknown ) do begin
    Case FBuff[ chIx ] of
      LF: InType := ftUnixText;  // Ran into LF first, should be UNIX
      CR: InType := ftDosText;   // Ran into CR, must be DOS
    end;
    Inc( chIx );
  end;
end;


Procedure Convert( InType, OutType: TxtFType;
                   const FBuff: PChar; const cbInBytes: LongInt;
                   var OutFile: File );
const
  outBufSize = 2048;
var
  cbWrt, cbOutbytes, cntLines, bufIx, cbBuff: LongInt;
  OutBuff: Array [1..outBufSize] of Char;
  
  Procedure WriteBuff( ch: char );
  begin
    if ch <> #4 then begin  // Discard EOT-flag
      Inc( cbBuff );
      OutBuff[ cbBuff ] := ch;
    end;
    if ( cbBuff = outBufSize ) or ( bufIx >= cbInBytes ) then begin
      BlockWrite( OutFile, OutBuff, cbBuff, cbWrt );
      cbBuff := 0;
    end
  end;

begin
  Write('CR/LF-convert ');
  Case InType of                 // Only these two values are possible!
    ftDosText:  write('DOS');
    ftUnixText: write('UNIX');
  end;
  write(' to ');
  Case OutType of                 // Only these two values are possible!
    ftDosText:  write('DOS ... ');
    ftUnixText: write('UNIX ... ');
  end;
  cbOutBytes := cbInBytes;
  cntLines := 0;
  cbBuff := 0;
  bufIx := 0;
  while bufIx < cbInBytes do begin
    Case FBuff[ bufIx ] of
      LF: begin       // If DOS-type send CR before LF
            if OutType = ftDosText then begin
              WriteBuff( CR );      // If DOS send CR here
              Inc( cbOutBytes );    // and count it
            end;
            WriteBuff( FBuff[ bufIx ]); // Always put LF through
            Inc( cntLines );         // Count lines
          end;
      CR: Dec( cbOutBytes );   // CR always suppressed here
      else WriteBuff( FBuff[ bufIx ] );  // Any other char always put through
    end;  { Case }
    Inc( bufIx );
  end;  { while }
  WriteBuff( #4 );
  writeln(cntLines, ' lines: ', cbInbytes, ' -> ', cbOutBytes, ' bytes.');
end;


(* MAIN *)
var
  InFile, OutFile: File;
  InFName, OutFName: String;
  FBuff: PChar;
  FSize, cbRead: LongInt;
  InType, OutType: TxtFType;

begin
  if ParamCount < 1 then ShowUsage
  else begin
      // Read program parameters
    ProsessParams( OutType, InFName, OutFName );
      // Read source file into buffer
    Assign( InFile, InFName );
    Reset( InFile, 1 );
    FSize := FileSize( InFile );
    GetMem( FBuff, FSize );
    BlockRead( InFile, FBuff[0], FSize, cbRead );
    if cbRead <> FSize then begin
      writeln('File-read error! Aborting.');
      Halt( 1 );
    end;
    Close( InFile );
      // Check what type source seem to be
    DetectType( InType, FBuff, FSize );
      // Did user make a filtype query?
    if OutType = ftQueried then begin    // YES - write detect result
      Write( '''', InFName, ''' is a ' );
      Case InType of
        ftUnixText: Write('UNIX');
        ftDosText: Write('DOS');
      end;
      Writeln('-type file.');
    end
    else begin                           // NO - convert fil
      if OutType = ftUnknown then
       if InType = ftDosText then OutType := ftUnixText
        else OutType := ftDosText;       // Will default to DOS
        // If no target fname, set target to source
      if OutFName = '' then OutFName := InFName;
      Assign( OutFile, OutFName );
      Rewrite( OutFile, 1 );
      Convert( InType, OutType, FBuff, FSize, OutFile );
      Close( OutFile );
    end;  { else }
    FreeMem( FBuff, FSize );
  end;  { else }
end.
